@@ -2,192 +2,38 @@
 # coding: utf-8
 """html2text: Turn HTML into equivalent Markdown-structured text."""
 from __future__ import division
-from html2text import config
-
-
-__version__ = "2014.9.25"
-
-
-# TODO:
-#   Support decoded entities with UNIFIABLE.
-
-try:
-    import htmlentitydefs
-    import urlparse
-    import HTMLParser
-except ImportError:  # Python3
-    import html.entities as htmlentitydefs
-    import urllib.parse as urlparse
-    import html.parser as HTMLParser
-try:  # Python3
-    import urllib.request as urllib
-except ImportError:
-    import urllib
-import optparse
 import re
-import sys
+import cgi
 
 try:
     from textwrap import wrap
-except ImportError:
+except ImportError:  # pragma: no cover
     pass
 
+from html2text.compat import urlparse, HTMLParser
+from html2text import config
 
-def name2cp(k):
-    if k == 'apos':
-        return ord("'")
-    return htmlentitydefs.name2codepoint[k]
+from html2text.utils import (
+    name2cp,
+    unifiable_n,
+    google_text_emphasis,
+    google_fixed_width_font,
+    element_style,
+    hn,
+    google_has_height,
+    escape_md,
+    google_list_style,
+    list_numbering_start,
+    dumb_css_parser,
+    escape_md_section,
+    skipwrap
+)
 
-unifiable_n = {}
-
-for k in config.UNIFIABLE.keys():
-    unifiable_n[name2cp(k)] = config.UNIFIABLE[k]
+__version__ = (2016, 1, 8)
 
 
-def hn(tag):
-    if tag[0] == 'h' and len(tag) == 2:
-        try:
-            n = int(tag[1])
-            if n in range(1, 10):
-                return n
-        except ValueError:
-            return 0
-
-
-def dumb_property_dict(style):
-    """
-    :returns: A hash of css attributes
-    """
-    out = dict([(x.strip(), y.strip()) for x, y in
-               [z.split(':', 1) for z in
-               style.split(';') if ':' in z]])
-    return out
-
-
-def dumb_css_parser(data):
-    """
-    :type data: str
-
-    :returns: A hash of css selectors, each of which contains a hash of
-    css attributes.
-    :rtype: dict
-    """
-    # remove @import sentences
-    data += ';'
-    importIndex = data.find('@import')
-    while importIndex != -1:
-        data = data[0:importIndex] + data[data.find(';', importIndex) + 1:]
-        importIndex = data.find('@import')
-
-    # parse the css. reverted from dictionary comprehension in order to
-    # support older pythons
-    elements = [x.split('{') for x in data.split('}') if '{' in x.strip()]
-    try:
-        elements = dict([(a.strip(), dumb_property_dict(b))
-                        for a, b in elements])
-    except ValueError:
-        elements = {}  # not that important
-
-    return elements
-
-
-def element_style(attrs, style_def, parent_style):
-    """
-    :type attrs: dict
-    :type style_def: dict
-    :type style_def: dict
-
-    :returns: A hash of the 'final' style attributes of the element
-    :rtype: dict
-    """
-    style = parent_style.copy()
-    if 'class' in attrs:
-        for css_class in attrs['class'].split():
-            css_style = style_def['.' + css_class]
-            style.update(css_style)
-    if 'style' in attrs:
-        immediate_style = dumb_property_dict(attrs['style'])
-        style.update(immediate_style)
-    return style
-
-
-def google_list_style(style):
-    """
-    Finds out whether this is an ordered or unordered list
-
-    :type style: dict
-
-    :rtype: str
-    """
-    if 'list-style-type' in style:
-        list_style = style['list-style-type']
-        if list_style in ['disc', 'circle', 'square', 'none']:
-            return 'ul'
-    return 'ol'
-
-
-def google_has_height(style):
-    """
-    Check if the style of the element has the 'height' attribute
-    explicitly defined
-
-    :type style: dict
-
-    :rtype: bool
-    """
-    if 'height' in style:
-        return True
-    return False
-
-
-def google_text_emphasis(style):
-    """
-    :type style: dict
-
-    :returns: A list of all emphasis modifiers of the element
-    :rtype: list
-    """
-    emphasis = []
-    if 'text-decoration' in style:
-        emphasis.append(style['text-decoration'])
-    if 'font-style' in style:
-        emphasis.append(style['font-style'])
-    if 'font-weight' in style:
-        emphasis.append(style['font-weight'])
-    return emphasis
-
-
-def google_fixed_width_font(style):
-    """
-    Check if the css of the current element defines a fixed width font
-
-    :type style: dict
-
-    :rtype: bool
-    """
-    font_family = ''
-    if 'font-family' in style:
-        font_family = style['font-family']
-    if 'Courier New' == font_family or 'Consolas' == font_family:
-        return True
-    return False
-
-
-def list_numbering_start(attrs):
-    """
-    Extract numbering from list element attributes
-
-    :type attrs: dict
-
-    :rtype: int or None
-    """
-    if 'start' in attrs:
-        try:
-            return int(attrs['start']) - 1
-        except ValueError:
-            pass
-
-    return 0
+# TODO:
+# Support decoded entities with UNIFIABLE.
 
 
 class HTML2Text(HTMLParser.HTMLParser):
@@ -204,25 +50,34 @@ class HTML2Text(HTMLParser.HTMLParser):
         self.split_next_td = False
         self.td_count = 0
         self.table_start = False
-        self.unicode_snob = config.UNICODE_SNOB
-        self.escape_snob = config.ESCAPE_SNOB
+        self.unicode_snob = config.UNICODE_SNOB  # covered in cli
+        self.escape_snob = config.ESCAPE_SNOB  # covered in cli
         self.links_each_paragraph = config.LINKS_EACH_PARAGRAPH
-        self.body_width = bodywidth
-        self.skip_internal_links = config.SKIP_INTERNAL_LINKS
-        self.inline_links = config.INLINE_LINKS
-        self.google_list_indent = config.GOOGLE_LIST_INDENT
-        self.ignore_links = config.IGNORE_ANCHORS
-        self.ignore_images = config.IGNORE_IMAGES
-        self.ignore_emphasis = config.IGNORE_EMPHASIS
-        self.bypass_tables = config.BYPASS_TABLES
-        self.google_doc = False
-        self.ul_item_mark = '*'
-        self.emphasis_mark = '_'
+        self.body_width = bodywidth  # covered in cli
+        self.skip_internal_links = config.SKIP_INTERNAL_LINKS  # covered in cli
+        self.inline_links = config.INLINE_LINKS  # covered in cli
+        self.protect_links = config.PROTECT_LINKS  # covered in cli
+        self.google_list_indent = config.GOOGLE_LIST_INDENT  # covered in cli
+        self.ignore_links = config.IGNORE_ANCHORS  # covered in cli
+        self.ignore_images = config.IGNORE_IMAGES  # covered in cli
+        self.images_to_alt = config.IMAGES_TO_ALT  # covered in cli
+        self.images_with_size = config.IMAGES_WITH_SIZE  # covered in cli
+        self.ignore_emphasis = config.IGNORE_EMPHASIS  # covered in cli
+        self.bypass_tables = config.BYPASS_TABLES  # covered in cli
+        self.google_doc = False  # covered in cli
+        self.ul_item_mark = '*'  # covered in cli
+        self.emphasis_mark = '_'  # covered in cli
         self.strong_mark = '**'
+        self.single_line_break = config.SINGLE_LINE_BREAK  # covered in cli
+        self.use_automatic_links = config.USE_AUTOMATIC_LINKS  # covered in cli
+        self.hide_strikethrough = False  # covered in cli
+        self.mark_code = config.MARK_CODE
+        self.wrap_links = config.WRAP_LINKS  # covered in cli
+        self.tag_callback = None
 
-        if out is None:
+        if out is None:  # pragma: no cover
             self.out = self.outtextf
-        else:
+        else:  # pragma: no cover
             self.out = out
 
         # empty list to store output characters before they are "joined"
@@ -236,6 +91,7 @@ class HTML2Text(HTMLParser.HTMLParser):
         self.a = []
         self.astack = []
         self.maybe_automatic_link = None
+        self.empty_link = False
         self.absolute_url_matcher = re.compile(r'^[a-zA-Z+]+://')
         self.acount = 0
         self.list = []
@@ -311,10 +167,17 @@ class HTML2Text(HTMLParser.HTMLParser):
         return outtext
 
     def handle_charref(self, c):
-        self.o(self.charref(c), 1)
+        charref = self.charref(c)
+        if not self.code and not self.pre:
+            charref = cgi.escape(charref)
+        self.handle_data(charref, True)
 
     def handle_entityref(self, c):
-        self.o(self.entityref(c), 1)
+        entityref = self.entityref(c)
+        if (not self.code and not self.pre
+                and entityref != '&nbsp_place_holder;'):
+            entityref = cgi.escape(entityref)
+        self.handle_data(entityref, True)
 
     def handle_starttag(self, tag, attrs):
         self.handle_tag(tag, attrs, 1)
@@ -330,9 +193,8 @@ class HTML2Text(HTMLParser.HTMLParser):
         self.a list. If the set of attributes is not found, returns None
         :rtype: int
         """
-        if 'href' not in attrs:
+        if 'href' not in attrs:  # pragma: no cover
             return None
-
         i = -1
         for a in self.a:
             i += 1
@@ -340,9 +202,9 @@ class HTML2Text(HTMLParser.HTMLParser):
 
             if ('href' in a) and a['href'] == attrs['href']:
                 if ('title' in a) or ('title' in attrs):
-                        if (('title' in a) and ('title' in attrs) and
+                    if (('title' in a) and ('title' in attrs) and
                                 a['title'] == attrs['title']):
-                            match = True
+                        match = True
                 else:
                     match = True
 
@@ -358,7 +220,7 @@ class HTML2Text(HTMLParser.HTMLParser):
 
         # handle Google's text emphasis
         strikethrough = 'line-through' in \
-            tag_emphasis and self.hide_strikethrough
+                        tag_emphasis and self.hide_strikethrough
         bold = 'bold' in tag_emphasis and not 'bold' in parent_emphasis
         italic = 'italic' in tag_emphasis and not 'italic' in parent_emphasis
         fixed = google_fixed_width_font(tag_style) and not \
@@ -407,7 +269,7 @@ class HTML2Text(HTMLParser.HTMLParser):
                     self.o(self.emphasis_mark)
             # space is only allowed after *all* emphasis marks
             if (bold or italic) and not self.emphasis:
-                    self.o(" ")
+                self.o(" ")
             if strikethrough:
                 self.quiet -= 1
 
@@ -418,6 +280,18 @@ class HTML2Text(HTMLParser.HTMLParser):
         else:
             attrs = dict(attrs)
 
+        if self.tag_callback is not None:
+            if self.tag_callback(self, tag, attrs, start) is True:
+                return
+
+        # first thing inside the anchor tag is another tag that produces some output
+        if (start and not self.maybe_automatic_link is None
+                and tag not in ['p', 'div', 'style', 'dl', 'dt']
+                and (tag != "img" or self.ignore_images)):
+            self.o("[")
+            self.maybe_automatic_link = None
+            self.empty_link = False
+
         if self.google_doc:
             # the attrs parameter is empty for a closing tag. in addition, we
             # need the attributes of the parent nodes in order to get a
@@ -430,7 +304,7 @@ class HTML2Text(HTMLParser.HTMLParser):
                 tag_style = element_style(attrs, self.style_def, parent_style)
                 self.tag_stack.append((tag, attrs, tag_style))
             else:
-                dummy, attrs, tag_style = self.tag_stack.pop()
+                dummy, attrs, tag_style = self.tag_stack.pop() if self.tag_stack else (None, {}, {})
                 if self.tag_stack:
                     parent_style = self.tag_stack[-1][2]
 
@@ -491,9 +365,9 @@ class HTML2Text(HTMLParser.HTMLParser):
             self.o(self.strong_mark)
         if tag in ['del', 'strike', 's']:
             if start:
-                self.o("<" + tag + ">")
+                self.o('~~')
             else:
-                self.o("</" + tag + ">")
+                self.o('~~')
 
         if self.google_doc:
             if not self.inheader:
@@ -502,6 +376,7 @@ class HTML2Text(HTMLParser.HTMLParser):
 
         if tag in ["code", "tt"] and not self.pre:
             self.o('`')  # TODO: `` `this` ``
+            self.code = not self.code
         if tag == "abbr":
             if start:
                 self.abbr_title = None
@@ -519,19 +394,32 @@ class HTML2Text(HTMLParser.HTMLParser):
                 if ('href' in attrs) and \
                         (attrs['href'] is not None) and \
                         not (self.skip_internal_links and
-                             attrs['href'].startswith('#')):
+                                 attrs['href'].startswith('#')):
                     self.astack.append(attrs)
                     self.maybe_automatic_link = attrs['href']
+                    self.empty_link = True
+                    if self.protect_links:
+                        attrs['href'] = '<'+attrs['href']+'>'
                 else:
                     self.astack.append(None)
             else:
                 if self.astack:
                     a = self.astack.pop()
-                    if self.maybe_automatic_link:
+                    if self.maybe_automatic_link and not self.empty_link:
                         self.maybe_automatic_link = None
                     elif a:
+                        if self.empty_link:
+                            self.o("[")
+                            self.empty_link = False
+                            self.maybe_automatic_link = None
                         if self.inline_links:
-                            self.o("](" + escape_md(a['href']) + ")")
+                            try:
+                                title = escape_md(a['title'])
+                            except KeyError:
+                                self.o("](" + escape_md(a['href']) + ")")
+                            else:
+                                self.o("](" + escape_md(a['href'])
+                                       + ' "' + title + '" )')
                         else:
                             i = self.previousIndex(a)
                             if i is not None:
@@ -545,23 +433,56 @@ class HTML2Text(HTMLParser.HTMLParser):
 
         if tag == "img" and start and not self.ignore_images:
             if 'src' in attrs:
-                attrs['href'] = attrs['src']
+                if not self.images_to_alt:
+                    attrs['href'] = attrs['src']
                 alt = attrs.get('alt') or ''
-                self.o("![" + escape_md(alt) + "]")
 
-                if self.inline_links:
-                    href = attrs.get('href') or ''
-                    self.o("(" + escape_md(href) + ")")
-                else:
-                    i = self.previousIndex(attrs)
-                    if i is not None:
-                        attrs = self.a[i]
+                # If we have images_with_size, write raw html including width,
+                # height, and alt attributes
+                if self.images_with_size and \
+                        ("width" in attrs or "height" in attrs):
+                    self.o("<img src='" + attrs["src"] + "' ")
+                    if "width" in attrs:
+                        self.o("width='" + attrs["width"] + "' ")
+                    if "height" in attrs:
+                        self.o("height='" + attrs["height"] + "' ")
+                    if alt:
+                        self.o("alt='" + alt + "' ")
+                    self.o("/>")
+                    return
+
+                # If we have a link to create, output the start
+                if not self.maybe_automatic_link is None:
+                    href = self.maybe_automatic_link
+                    if self.images_to_alt and escape_md(alt) == href and \
+                            self.absolute_url_matcher.match(href):
+                        self.o("<" + escape_md(alt) + ">")
+                        self.empty_link = False
+                        return
                     else:
-                        self.acount += 1
-                        attrs['count'] = self.acount
-                        attrs['outcount'] = self.outcount
-                        self.a.append(attrs)
-                    self.o("[" + str(attrs['count']) + "]")
+                        self.o("[")
+                        self.maybe_automatic_link = None
+                        self.empty_link = False
+
+                # If we have images_to_alt, we discard the image itself,
+                # considering only the alt text.
+                if self.images_to_alt:
+                    self.o(escape_md(alt))
+                else:
+                    self.o("![" + escape_md(alt) + "]")
+                    if self.inline_links:
+                        href = attrs.get('href') or ''
+                        self.o("(" + escape_md(href) + ")")
+                    else:
+                        i = self.previousIndex(attrs)
+                        if i is not None:
+                            attrs = self.a[i]
+                        else:
+                            self.acount += 1
+                            attrs['count'] = self.acount
+                            attrs['outcount'] = self.outcount
+                            self.a.append(attrs)
+                        self.o("[" + str(attrs['count']) + "]")
 
         if tag == 'dl' and start:
             self.p()
@@ -655,16 +576,22 @@ class HTML2Text(HTMLParser.HTMLParser):
                 self.pre = 1
             else:
                 self.pre = 0
+                if self.mark_code:
+                    self.out("\n[/code]")
             self.p()
 
+    # TODO: Add docstring for these one letter functions
     def pbr(self):
+        "Pretty print has a line break"
         if self.p_p == 0:
             self.p_p = 1
 
     def p(self):
-        self.p_p = 2
+        "Set pretty print to 1 or 2 lines"
+        self.p_p = 1 if self.single_line_break else 2
 
     def soft_br(self):
+        "Soft breaks"
         self.pbr()
         self.br_toggle = '  '
 
@@ -700,6 +627,9 @@ class HTML2Text(HTMLParser.HTMLParser):
                 #self.out(" :") #TODO: not output when already one there
                 if not data.startswith("\n"):  # <pre>stuff...
                     data = "\n" + data
+                if self.mark_code:
+                    self.out("\n[code]")
+                    self.p_p = 0
 
             bq = (">" * self.blockquote)
             if not (force and data and data[0] == ">") and self.blockquote:
@@ -770,7 +700,7 @@ class HTML2Text(HTMLParser.HTMLParser):
             self.out(data)
             self.outcount += 1
 
-    def handle_data(self, data):
+    def handle_data(self, data, entity_char=False):
         if r'\/script>' in data:
             self.quiet -= 1
 
@@ -779,18 +709,22 @@ class HTML2Text(HTMLParser.HTMLParser):
 
         if not self.maybe_automatic_link is None:
             href = self.maybe_automatic_link
-            if href == data and self.absolute_url_matcher.match(href):
+            if (href == data and self.absolute_url_matcher.match(href)
+                    and self.use_automatic_links):
                 self.o("<" + data + ">")
+                self.empty_link = False
                 return
             else:
                 self.o("[")
                 self.maybe_automatic_link = None
+                self.empty_link = False
 
-        if not self.code and not self.pre:
+        if not self.code and not self.pre and not entity_char:
             data = escape_md_section(data, snob=self.escape_snob)
         self.o(data, 1)
 
-    def unknown_decl(self, data):
+    def unknown_decl(self, data):  # pragma: no cover
+        # TODO: what is this doing here?
         pass
 
     def charref(self, name):
@@ -803,9 +737,12 @@ class HTML2Text(HTMLParser.HTMLParser):
             return unifiable_n[c]
         else:
             try:
-                return unichr(c)
-            except NameError:  # Python3
-                return chr(c)
+                try:
+                    return unichr(c)
+                except NameError:  # Python3
+                    return chr(c)
+            except ValueError:  # invalid unicode
+                return ''
 
     def entityref(self, c):
         if not self.unicode_snob and c in config.UNIFIABLE.keys():
@@ -845,7 +782,7 @@ class HTML2Text(HTMLParser.HTMLParser):
         nest_count = 0
         if 'margin-left' in style:
             nest_count = int(style['margin-left'][:-2]) \
-                // self.google_list_indent
+                         // self.google_list_indent
 
         return nest_count
 
@@ -863,9 +800,14 @@ class HTML2Text(HTMLParser.HTMLParser):
         assert wrap, "Requires Python 2.3."
         result = ''
         newlines = 0
+        # I cannot think of a better solution for now.
+        # To avoid the non-wrap behaviour for entire paras
+        # because of the presence of a link in it
+        if not self.wrap_links:
+            self.inline_links = False
         for para in text.split("\n"):
             if len(para) > 0:
-                if not skipwrap(para):
+                if not skipwrap(para, self.wrap_links):
                     result += "\n".join(wrap(para, self.body_width))
                     if para.endswith('  '):
                         result += "  \n"
@@ -888,43 +830,9 @@ class HTML2Text(HTMLParser.HTMLParser):
         return result
 
 
-def skipwrap(para):
-    # If the text begins with four spaces or one tab, it's a code block;
-    # don't wrap
-    if para[0:4] == '    ' or para[0] == '\t':
-        return True
-
-    # If the text begins with only two "--", possibly preceded by
-    # whitespace, that's an emdash; so wrap.
-    stripped = para.lstrip()
-    if stripped[0:2] == "--" and len(stripped) > 2 and stripped[2] != "-":
-        return False
-
-    # I'm not sure what this is for; I thought it was to detect lists,
-    # but there's a <br>-inside-<span> case in one of the tests that
-    # also depends upon it.
-    if stripped[0:1] == '-' or stripped[0:1] == '*':
-        return True
-
-    # If the text begins with a single -, *, or +, followed by a space,
-    # or an integer, followed by a ., followed by a space (in either
-    # case optionally proceeded by whitespace), it's a list; don't wrap.
-    if config.RE_ORDERED_LIST_MATCHER.match(stripped) or \
-            config.RE_UNORDERED_LIST_MATCHER.match(stripped):
-        return True
-
-    return False
-
-
-def wrapwrite(text):
-    text = text.encode('utf-8')
-    try:  # Python3
-        sys.stdout.buffer.write(text)
-    except AttributeError:
-        sys.stdout.write(text)
-
-
-def html2text(html, baseurl='', bodywidth=config.BODY_WIDTH):
+def html2text(html, baseurl='', bodywidth=None):
+    if bodywidth is None:
+        bodywidth = config.BODY_WIDTH
     h = HTML2Text(baseurl=baseurl, bodywidth=bodywidth)
 
     return h.handle(html)
@@ -937,170 +845,7 @@ def unescape(s, unicode_snob=False):
     return h.unescape(s)
 
 
-def escape_md(text):
-    """
-    Escapes markdown-sensitive characters within other markdown
-    constructs.
-    """
-    return config.RE_MD_CHARS_MATCHER.sub(r"\\\1", text)
-
-
-def escape_md_section(text, snob=False):
-    """
-    Escapes markdown-sensitive characters across whole document sections.
-    """
-    text = config.RE_MD_BACKSLASH_MATCHER.sub(r"\\\1", text)
-
-    if snob:
-        text = config.RE_MD_CHARS_MATCHER_ALL.sub(r"\\\1", text)
-
-    text = config.RE_MD_DOT_MATCHER.sub(r"\1\\\2", text)
-    text = config.RE_MD_PLUS_MATCHER.sub(r"\1\\\2", text)
-    text = config.RE_MD_DASH_MATCHER.sub(r"\1\\\2", text)
-
-    return text
-
-
-def main():
-    baseurl = ''
-
-    p = optparse.OptionParser('%prog [(filename|url) [encoding]]',
-                              version='%prog ' + __version__)
-    p.add_option(
-        "--ignore-emphasis",
-        dest="ignore_emphasis",
-        action="store_true",
-        default=config.IGNORE_EMPHASIS,
-        help="don't include any formatting for emphasis"
-    )
-    p.add_option(
-        "--ignore-links",
-        dest="ignore_links",
-        action="store_true",
-        default=config.IGNORE_ANCHORS,
-        help="don't include any formatting for links")
-    p.add_option(
-        "--ignore-images",
-        dest="ignore_images",
-        action="store_true",
-        default=config.IGNORE_IMAGES,
-        help="don't include any formatting for images"
-    )
-    p.add_option(
-        "-g", "--google-doc",
-        action="store_true",
-        dest="google_doc",
-        default=False,
-        help="convert an html-exported Google Document"
-    )
-    p.add_option(
-        "-d", "--dash-unordered-list",
-        action="store_true",
-        dest="ul_style_dash",
-        default=False,
-        help="use a dash rather than a star for unordered list items"
-    )
-    p.add_option(
-        "-e", "--asterisk-emphasis",
-        action="store_true",
-        dest="em_style_asterisk",
-        default=False,
-        help="use an asterisk rather than an underscore for emphasized text"
-    )
-    p.add_option(
-        "-b", "--body-width",
-        dest="body_width",
-        action="store",
-        type="int",
-        default=config.BODY_WIDTH,
-        help="number of characters per output line, 0 for no wrap"
-    )
-    p.add_option(
-        "-i", "--google-list-indent",
-        dest="list_indent",
-        action="store",
-        type="int",
-        default=config.GOOGLE_LIST_INDENT,
-        help="number of pixels Google indents nested lists"
-    )
-    p.add_option(
-        "-s", "--hide-strikethrough",
-        action="store_true",
-        dest="hide_strikethrough",
-        default=False,
-        help="hide strike-through text. only relevant when -g is "
-             "specified as well"
-    )
-    p.add_option(
-        "--escape-all",
-        action="store_true",
-        dest="escape_snob",
-        default=False,
-        help="Escape all special characters.  Output is less readable, but "
-             "avoids corner case formatting issues."
-    )
-    p.add_option(
-        "--bypass-tables",
-        action="store_true",
-        dest="bypass_tables",
-        default=config.BYPASS_TABLES,
-        help="Format tables in HTML rather than Markdown syntax."
-    )
-    (options, args) = p.parse_args()
-
-    # process input
-    encoding = "utf-8"
-    if len(args) > 0:
-        file_ = args[0]
-        if len(args) == 2:
-            encoding = args[1]
-        if len(args) > 2:
-            p.error('Too many arguments')
-
-        if file_.startswith('http://') or file_.startswith('https://'):
-            baseurl = file_
-            j = urllib.urlopen(baseurl)
-            data = j.read()
-            if encoding is None:
-                try:
-                    from feedparser import _getCharacterEncoding as enc
-                except ImportError:
-                    enc = lambda x, y: ('utf-8', 1)
-                encoding = enc(j.headers, data)[0]
-                if encoding == 'us-ascii':
-                    encoding = 'utf-8'
-        else:
-            data = open(file_, 'rb').read()
-            if encoding is None:
-                try:
-                    from chardet import detect
-                except ImportError:
-                    detect = lambda x: {'encoding': 'utf-8'}
-                encoding = detect(data)['encoding']
-    else:
-        data = sys.stdin.read()
-
-    data = data.decode(encoding)
-    h = HTML2Text(baseurl=baseurl)
-    # handle options
-    if options.ul_style_dash:
-        h.ul_item_mark = '-'
-    if options.em_style_asterisk:
-        h.emphasis_mark = '*'
-        h.strong_mark = '__'
-
-    h.body_width = options.body_width
-    h.list_indent = options.list_indent
-    h.ignore_emphasis = options.ignore_emphasis
-    h.ignore_links = options.ignore_links
-    h.ignore_images = options.ignore_images
-    h.google_doc = options.google_doc
-    h.hide_strikethrough = options.hide_strikethrough
-    h.escape_snob = options.escape_snob
-    h.bypass_tables = options.bypass_tables
-
-    wrapwrite(h.handle(data))
-
-
 if __name__ == "__main__":
+    from html2text.cli import main
+
     main()
